@@ -1331,11 +1331,16 @@ main(int argc, char *argv[])
 	for (i = 1; i < argc; i++) {
 		if (strcmp("--click-to-show", argv[i]) == 0)
 			click_to_show = 1;
-		else if (strcmp("--preferred-language", argv[i]) == 0) {
-			if (i + 1 < argc) {
-				preferred_language = argv[i + 1];
-				i++;
-			}
+		else if (strcmp("--preferred-language", argv[i]) == 0 &&
+			 i + 1 < argc) {
+			preferred_language = argv[i + 1];
+			i++;
+		} else {
+			printf("Usage: %s [OPTIONS]\n"
+			       "  --click-to-show\n"
+			       "  --preferred-language LANGUAGE\n",
+			       argv[0]);
+			return 1;
 		}
 	}
 
@@ -1354,6 +1359,11 @@ main(int argc, char *argv[])
 	display_set_user_data(editor.display, &editor);
 	display_set_global_handler(editor.display, global_handler);
 
+	if (editor.text_input_manager == NULL) {
+		fprintf(stderr, "No text input manager global\n");
+		return -1;
+	}
+
 	editor.window = window_create(editor.display);
 	editor.widget = window_frame_create(editor.window, &editor);
 
@@ -1382,6 +1392,9 @@ main(int argc, char *argv[])
 
 	text_entry_destroy(editor.entry);
 	text_entry_destroy(editor.editor);
+	widget_destroy(editor.widget);
+	window_destroy(editor.window);
+	display_destroy(editor.display);
 
 	return 0;
 }
