@@ -20,13 +20,14 @@
  * OF THIS SOFTWARE.
  */
 
+#include "config.h"
+
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <math.h>
-#include <time.h>
 #include <cairo.h>
 #include <sys/time.h>
 
@@ -129,13 +130,6 @@ redraw_handler(struct widget *widget, void *data)
 	cairo_surface_destroy(surface);
 }
 
-static int
-motion_handler(struct widget *widget, struct input *input,
-	       uint32_t time, float x, float y, void *data)
-{
-	return CURSOR_HAND1;
-}
-
 static void
 button_handler(struct widget *widget,
 	       struct input *input, uint32_t time,
@@ -160,13 +154,22 @@ button_handler(struct widget *widget,
 	}
 }
 
+static void
+touch_down_handler(struct widget *widget, struct input *input, 
+		   uint32_t serial, uint32_t time, int32_t id, 
+		   float x, float y, void *data)
+{
+	struct flower *flower = data;
+	window_move(flower->window, input, display_get_serial(flower->display));
+}
+
 int main(int argc, char *argv[])
 {
 	struct flower flower;
 	struct display *d;
 	struct timeval tv;
 
-	d = display_create(argc, argv);
+	d = display_create(&argc, argv);
 	if (d == NULL) {
 		fprintf(stderr, "failed to create display: %m\n");
 		return -1;
@@ -184,12 +187,17 @@ int main(int argc, char *argv[])
 
 	widget_set_resize_handler(flower.widget, resize_handler);
 	widget_set_redraw_handler(flower.widget, redraw_handler);
-	widget_set_motion_handler(flower.widget, motion_handler);
 	widget_set_button_handler(flower.widget, button_handler);
+	widget_set_default_cursor(flower.widget, CURSOR_HAND1);
+	widget_set_touch_down_handler(flower.widget, touch_down_handler);
 
 	window_schedule_resize(flower.window, flower.width, flower.height);
 
 	display_run(d);
 
+	widget_destroy(flower.widget);
+	window_destroy(flower.window);
+	display_destroy(d);
+
 	return 0;
 }
