@@ -550,13 +550,6 @@ surface_set_input_region(struct wl_client *client,
 	fprintf(stderr, "surface_set_input_region\n");
 }
 
-static void
-empty_region(pixman_region32_t *region)
-{
-	pixman_region32_fini(region);
-	pixman_region32_init(region);
-}
-
 static void
 surface_commit(struct wl_client *client, struct wl_resource *resource)
 {
@@ -574,7 +567,7 @@ surface_commit(struct wl_client *client, struct wl_resource *resource)
 	surface->pending.newly_attached = 0;
 
 	/* wl_surface.damage */
-	empty_region(&surface->pending.damage);
+	pixman_region32_clear(&surface->pending.damage);
 
 	/* wl_surface.frame */
 	wl_list_insert_list(&surface->frame_callback_list,
@@ -1122,8 +1115,11 @@ main(int argc, char *argv[])
 	struct display *display;
 	struct nested *nested;
 
-	parse_options(nested_options,
-		      ARRAY_LENGTH(nested_options), &argc, argv);
+	if (parse_options(nested_options,
+			  ARRAY_LENGTH(nested_options), &argc, argv) > 1) {
+		printf("Usage: %s [OPTIONS]\n  --blit or -b\n", argv[0]);
+		exit(1);
+	}
 
 	display = display_create(&argc, argv);
 	if (display == NULL) {
