@@ -20,6 +20,9 @@
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "config.h"
+
+#include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,7 +32,10 @@
 
 #include "../shared/os-compatibility.h"
 
+#define MIN(x,y) (((x) < (y)) ? (x) : (y))
+
 typedef void (*print_info_t)(void *info);
+typedef void (*destroy_info_t)(void *info);
 
 struct global_info {
 	struct wl_list link;
@@ -39,6 +45,7 @@ struct global_info {
 	char *interface;
 
 	print_info_t print;
+	destroy_info_t destroy;
 };
 
 struct output_mode {
@@ -82,8 +89,13 @@ struct shm_info {
 struct seat_info {
 	struct global_info global;
 	struct wl_seat *seat;
+	struct weston_info *info;
 
 	uint32_t capabilities;
+	char *name;
+
+	int32_t repeat_rate;
+	int32_t repeat_delay;
 };
 
 struct weston_info {
@@ -94,6 +106,35 @@ struct weston_info {
 	bool roundtrip_needed;
 };
 
+static void *
+fail_on_null(void *p)
+{
+	if (p == NULL) {
+		fprintf(stderr, "%s: out of memory\n", program_invocation_short_name);
+		exit(EXIT_FAILURE);
+	}
+
+	return p;
+}
+
+static void *
+xmalloc(size_t s)
+{
+	return fail_on_null(malloc(s));
+}
+
+static void *
+xzalloc(size_t s)
+{
+	return fail_on_null(calloc(1, s));
+}
+
+static char *
+xstrdup(const char *s)
+{
+	return fail_on_null(strdup(s));
+}
+
 static void
 print_global_info(void *data)
 {
@@ -110,7 +151,7 @@ init_global_info(struct weston_info *info,
 {
 	global->id = id;
 	global->version = version;
-	global->interface = strdup(interface);
+	global->interface = xstrdup(interface);
 
 	wl_list_insert(info->infos.prev, &global->link);
 }
@@ -190,7 +231,7 @@ print_output_info(void *data)
 	       output->geometry.physical_height);
 	printf("\tmake: '%s', model: '%s',\n",
 	       output->geometry.make, output->geometry.model);
-	printf("\tsubpixel_orientation: %s, output_tranform: %s,\n",
+	printf("\tsubpixel_orientation: %s, output_transform: %s,\n",
 	       subpixel_orientation, transform);
 
 	wl_list_for_each(mode, &output->modes, link) {
@@ -220,8 +261,20 @@ print_shm_info(void *data)
 	printf("\tformats:");
 
 	wl_list_for_each(format, &shm->formats, link)
-		printf(" %s", (format->format == WL_SHM_FORMAT_ARGB8888) ?
-			      "ARGB8888" : "XRGB8888");
+		switch (format->format) {
+		case WL_SHM_FORMAT_ARGB8888:
+			printf(" ARGB8888");
+			break;
+		case WL_SHM_FORMAT_XRGB8888:
+			printf(" XRGB8888");
+			break;
+		case WL_SHM_FORMAT_RGB565:
+			printf(" RGB565");
+			break;
+		default:
+			printf(" unknown(%08x)", format->format);
+			break;
+		}
 
 	printf("\n");
 }
@@ -233,6 +286,7 @@ print_seat_info(void *data)
 
 	print_global_info(data);
 
+	printf("\tname: %s\n", seat->name);
 	printf("\tcapabilities:");
 
 	if (seat->capabilities & WL_SEAT_CAPABILITY_POINTER)
@@ -243,32 +297,134 @@ print_seat_info(void *data)
 		printf(" touch");
 
 	printf("\n");
+
+	if (seat->repeat_rate > 0)
+		printf("\tkeyboard repeat rate: %d\n", seat->repeat_rate);
+	if (seat->repeat_delay > 0)
+		printf("\tkeyboard repeat delay: %d\n", seat->repeat_delay);
 }
 
+static void
+keyboard_handle_keymap(void *data, struct wl_keyboard *keyboard,
+		       uint32_t format, int fd, uint32_t size)
+{
+}
+
+static void
+keyboard_handle_enter(void *data, struct wl_keyboard *keyboard,
+		      uint32_t serial, struct wl_surface *surface,
+		      struct wl_array *keys)
+{
+}
+
+static void
+keyboard_handle_leave(void *data, struct wl_keyboard *keyboard,
+		      uint32_t serial, struct wl_surface *surface)
+{
+}
+
+static void
+keyboard_handle_key(void *data, struct wl_keyboard *keyboard,
+		    uint32_t serial, uint32_t time, uint32_t key,
+		    uint32_t state)
+{
+}
+
+static void
+keyboard_handle_modifiers(void *data, struct wl_keyboard *keyboard,
+			  uint32_t serial, uint32_t mods_depressed,
+			  uint32_t mods_latched, uint32_t mods_locked,
+			  uint32_t group)
+{
+}
+
+static void
+keyboard_handle_repeat_info(void *data, struct wl_keyboard *keyboard,
+			    int32_t rate, int32_t delay)
+{
+	struct seat_info *seat = data;
+
+	seat->repeat_rate = rate;
+	seat->repeat_delay = delay;
+}
+
+static const struct wl_keyboard_listener keyboard_listener = {
+	keyboard_handle_keymap,
+	keyboard_handle_enter,
+	keyboard_handle_leave,
+	keyboard_handle_key,
+	keyboard_handle_modifiers,
+	keyboard_handle_repeat_info,
+};
+
 static void
 seat_handle_capabilities(void *data, struct wl_seat *wl_seat,
 			 enum wl_seat_capability caps)
 {
 	struct seat_info *seat = data;
+
 	seat->capabilities = caps;
+
+	/* we want listen for repeat_info from wl_keyboard, but only
+	 * do so if the seat info is >= 4 and if we actually have a
+	 * keyboard */
+	if (seat->global.version < 4)
+		return;
+
+	if (caps & WL_SEAT_CAPABILITY_KEYBOARD) {
+		struct wl_keyboard *keyboard;
+
+		keyboard = wl_seat_get_keyboard(seat->seat);
+		wl_keyboard_add_listener(keyboard, &keyboard_listener,
+					 seat);
+
+		seat->info->roundtrip_needed = true;
+	}
+}
+
+static void
+seat_handle_name(void *data, struct wl_seat *wl_seat,
+		 const char *name)
+{
+	struct seat_info *seat = data;
+	seat->name = xstrdup(name);
 }
 
 static const struct wl_seat_listener seat_listener = {
 	seat_handle_capabilities,
+	seat_handle_name,
 };
 
+static void
+destroy_seat_info(void *data)
+{
+	struct seat_info *seat = data;
+
+	wl_seat_destroy(seat->seat);
+
+	if (seat->name != NULL)
+		free(seat->name);
+}
+
 static void
 add_seat_info(struct weston_info *info, uint32_t id, uint32_t version)
 {
-	struct seat_info *seat = malloc(sizeof *seat);
+	struct seat_info *seat = xzalloc(sizeof *seat);
+
+	/* required to set roundtrip_needed to true in capabilities
+	 * handler */
+	seat->info = info;
 
 	init_global_info(info, &seat->global, id, "wl_seat", version);
 	seat->global.print = print_seat_info;
+	seat->global.destroy = destroy_seat_info;
 
 	seat->seat = wl_registry_bind(info->registry,
-				      id, &wl_seat_interface, 1);
+				      id, &wl_seat_interface, MIN(version, 4));
 	wl_seat_add_listener(seat->seat, &seat_listener, seat);
 
+	seat->repeat_rate = seat->repeat_delay = -1;
+
 	info->roundtrip_needed = true;
 }
 
@@ -276,7 +432,7 @@ static void
 shm_handle_format(void *data, struct wl_shm *wl_shm, uint32_t format)
 {
 	struct shm_info *shm = data;
-	struct shm_format *shm_format = malloc(sizeof *shm_format);
+	struct shm_format *shm_format = xzalloc(sizeof *shm_format);
 
 	wl_list_insert(&shm->formats, &shm_format->link);
 	shm_format->format = format;
@@ -286,13 +442,29 @@ static const struct wl_shm_listener shm_listener = {
 	shm_handle_format,
 };
 
+static void
+destroy_shm_info(void *data)
+{
+	struct shm_info *shm = data;
+	struct shm_format *format, *tmp;
+
+	wl_list_for_each_safe(format, tmp, &shm->formats, link) {
+		wl_list_remove(&format->link);
+		free(format);
+	}
+
+	wl_shm_destroy(shm->shm);
+}
+
 static void
 add_shm_info(struct weston_info *info, uint32_t id, uint32_t version)
 {
-	struct shm_info *shm = malloc(sizeof *shm);
+	struct shm_info *shm = xzalloc(sizeof *shm);
 
 	init_global_info(info, &shm->global, id, "wl_shm", version);
 	shm->global.print = print_shm_info;
+	shm->global.destroy = destroy_shm_info;
+
 	wl_list_init(&shm->formats);
 
 	shm->shm = wl_registry_bind(info->registry,
@@ -317,8 +489,8 @@ output_handle_geometry(void *data, struct wl_output *wl_output,
 	output->geometry.physical_width = physical_width;
 	output->geometry.physical_height = physical_height;
 	output->geometry.subpixel = subpixel;
-	output->geometry.make = strdup(make);
-	output->geometry.model = strdup(model);
+	output->geometry.make = xstrdup(make);
+	output->geometry.model = xstrdup(model);
 	output->geometry.output_transform = output_transform;
 }
 
@@ -328,7 +500,7 @@ output_handle_mode(void *data, struct wl_output *wl_output,
 		   int32_t refresh)
 {
 	struct output_info *output = data;
-	struct output_mode *mode = malloc(sizeof *mode);
+	struct output_mode *mode = xmalloc(sizeof *mode);
 
 	mode->flags = flags;
 	mode->width = width;
@@ -343,13 +515,33 @@ static const struct wl_output_listener output_listener = {
 	output_handle_mode,
 };
 
+static void
+destroy_output_info(void *data)
+{
+	struct output_info *output = data;
+	struct output_mode *mode, *tmp;
+
+	wl_output_destroy(output->output);
+
+	if (output->geometry.make != NULL)
+		free(output->geometry.make);
+	if (output->geometry.model != NULL)
+		free(output->geometry.model);
+
+	wl_list_for_each_safe(mode, tmp, &output->modes, link) {
+		wl_list_remove(&mode->link);
+		free(mode);
+	}
+}
+
 static void
 add_output_info(struct weston_info *info, uint32_t id, uint32_t version)
 {
-	struct output_info *output = malloc(sizeof *output);
+	struct output_info *output = xzalloc(sizeof *output);
 
 	init_global_info(info, &output->global, id, "wl_output", version);
 	output->global.print = print_output_info;
+	output->global.destroy = destroy_output_info;
 
 	wl_list_init(&output->modes);
 
@@ -361,14 +553,20 @@ add_output_info(struct weston_info *info, uint32_t id, uint32_t version)
 	info->roundtrip_needed = true;
 }
 
+static void
+destroy_global_info(void *data)
+{
+}
+
 static void
 add_global_info(struct weston_info *info, uint32_t id,
 		const char *interface, uint32_t version)
 {
-	struct global_info *global = malloc(sizeof *global);
+	struct global_info *global = xzalloc(sizeof *global);
 
 	init_global_info(info, global, id, interface, version);
 	global->print = print_global_info;
+	global->destroy = destroy_global_info;
 }
 
 static void
@@ -387,8 +585,14 @@ global_handler(void *data, struct wl_registry *registry, uint32_t id,
 		add_global_info(info, id, interface, version);
 }
 
+static void
+global_remove_handler(void *data, struct wl_registry *registry, uint32_t name)
+{
+}
+
 static const struct wl_registry_listener registry_listener = {
-	global_handler
+	global_handler,
+	global_remove_handler
 };
 
 static void
@@ -400,6 +604,25 @@ print_infos(struct wl_list *infos)
 		info->print(info);
 }
 
+static void
+destroy_info(void *data)
+{
+	struct global_info *global = data;
+
+	global->destroy(data);
+	wl_list_remove(&global->link);
+	free(global->interface);
+	free(data);
+}
+
+static void
+destroy_infos(struct wl_list *infos)
+{
+	struct global_info *info, *tmp;
+	wl_list_for_each_safe(info, tmp, infos, link)
+		destroy_info(info);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -422,6 +645,10 @@ main(int argc, char **argv)
 	} while (info.roundtrip_needed);
 
 	print_infos(&info.infos);
+	destroy_infos(&info.infos);
+
+	wl_registry_destroy(info.registry);
+	wl_display_disconnect(info.display);
 
 	return 0;
 }
