@@ -200,7 +200,7 @@ create_wscreensaver_instance(struct wscreensaver *screensaver,
 					window_get_wl_surface(mi->window),
 					output);
 	} else {
-		mi->widget = frame_create(mi->window, mi);
+		mi->widget = window_frame_create(mi->window, mi);
 	}
 	widget_set_redraw_handler(mi->widget, redraw_handler);
 
@@ -310,10 +310,14 @@ int main(int argc, char *argv[])
 
 	init_frand();
 
-	argc = parse_options(wscreensaver_options,
-			     ARRAY_LENGTH(wscreensaver_options), argc, argv);
+	if (parse_options(wscreensaver_options,
+			ARRAY_LENGTH(wscreensaver_options), &argc, argv) > 1) {
+		printf("Usage: %s [OPTIONS]\n  --demo for demo mode\n",
+		       argv[0]);
+		exit(1);
+	}
 
-	d = display_create(argc, argv);
+	d = display_create(&argc, argv);
 	if (d == NULL) {
 		fprintf(stderr, "failed to create display: %m\n");
 		return EXIT_FAILURE;
