@@ -262,7 +262,7 @@ fs_output_create(struct fullscreen_shell *shell, struct weston_output *output)
 	fsout->black_view = create_black_surface(shell->compositor, fsout,
 						 output->x, output->y,
 						 output->width, output->height);
-	wl_list_insert(&shell->layer.view_list,
+	weston_layer_entry_insert(&shell->layer.view_list,
 		       &fsout->black_view->layer_link);
 	wl_list_init(&fsout->transform.link);
 	return fsout;
@@ -573,7 +573,7 @@ fs_output_apply_pending(struct fs_output *fsout)
 
 		wl_signal_add(&fsout->surface->destroy_signal,
 			      &fsout->surface_destroyed);
-		wl_list_insert(&fsout->shell->layer.view_list,
+		weston_layer_entry_insert(&fsout->shell->layer.view_list,
 			       &fsout->view->layer_link);
 	}
 
