@@ -20,12 +20,16 @@
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "config.h"
+
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <linux/input.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <mtdev.h>
+#include <assert.h>
 
 #include "compositor.h"
 #include "evdev.h"
@@ -43,10 +47,10 @@ evdev_led_update(struct evdev_device *device, enum weston_led leds)
 		{ LED_CAPS_LOCK, LED_CAPSL },
 		{ LED_SCROLL_LOCK, LED_SCROLLL },
 	};
-	struct input_event ev[ARRAY_LENGTH(map)];
+	struct input_event ev[ARRAY_LENGTH(map) + 1];
 	unsigned int i;
 
-	if (!device->caps & EVDEV_KEYBOARD)
+	if (!(device->seat_caps & EVDEV_SEAT_KEYBOARD))
 		return;
 
 	memset(ev, 0, sizeof(ev));
@@ -55,17 +59,143 @@ evdev_led_update(struct evdev_device *device, enum weston_led leds)
 		ev[i].code = map[i].evdev;
 		ev[i].value = !!(leds & map[i].weston);
 	}
+	ev[i].type = EV_SYN;
+	ev[i].code = SYN_REPORT;
 
 	i = write(device->fd, ev, sizeof ev);
 	(void)i; /* no, we really don't care about the return value */
 }
 
+static void
+transform_absolute(struct evdev_device *device, int32_t *x, int32_t *y)
+{
+       if (!device->abs.apply_calibration) {
+               *x = device->abs.x;
+               *y = device->abs.y;
+               return;
+       } else {
+               *x = device->abs.x * device->abs.calibration[0] +
+                       device->abs.y * device->abs.calibration[1] +
+                       device->abs.calibration[2];
+
+               *y = device->abs.x * device->abs.calibration[3] +
+                       device->abs.y * device->abs.calibration[4] +
+                       device->abs.calibration[5];
+       }
+}
+
+static void
+evdev_flush_pending_event(struct evdev_device *device, uint32_t time)
+{
+	struct weston_seat *master = device->seat;
+	wl_fixed_t x, y;
+	int32_t cx, cy;
+	int slot, seat_slot;
+
+	slot = device->mt.slot;
+	switch (device->pending_event) {
+	case EVDEV_NONE:
+		return;
+	case EVDEV_RELATIVE_MOTION:
+		notify_motion(master, time, device->rel.dx, device->rel.dy);
+		device->rel.dx = 0;
+		device->rel.dy = 0;
+		break;
+	case EVDEV_ABSOLUTE_MT_DOWN:
+		if (device->output == NULL)
+			break;
+		weston_output_transform_coordinate(device->output,
+						   wl_fixed_from_int(device->mt.slots[slot].x),
+						   wl_fixed_from_int(device->mt.slots[slot].y),
+						   &x, &y);
+		seat_slot = ffs(~master->slot_map) - 1;
+		device->mt.slots[slot].seat_slot = seat_slot;
+		master->slot_map |= 1 << seat_slot;
+
+		notify_touch(master, time, seat_slot, x, y, WL_TOUCH_DOWN);
+		break;
+	case EVDEV_ABSOLUTE_MT_MOTION:
+		if (device->output == NULL)
+			break;
+		weston_output_transform_coordinate(device->output,
+						   wl_fixed_from_int(device->mt.slots[slot].x),
+						   wl_fixed_from_int(device->mt.slots[slot].y),
+						   &x, &y);
+		seat_slot = device->mt.slots[slot].seat_slot;
+		notify_touch(master, time, seat_slot, x, y, WL_TOUCH_MOTION);
+		break;
+	case EVDEV_ABSOLUTE_MT_UP:
+		seat_slot = device->mt.slots[slot].seat_slot;
+		master->slot_map &= ~(1 << seat_slot);
+		notify_touch(master, time, seat_slot, 0, 0, WL_TOUCH_UP);
+		break;
+	case EVDEV_ABSOLUTE_TOUCH_DOWN:
+		if (device->output == NULL)
+			break;
+		transform_absolute(device, &cx, &cy);
+		weston_output_transform_coordinate(device->output,
+						   wl_fixed_from_int(cx),
+						   wl_fixed_from_int(cy),
+						   &x, &y);
+		seat_slot = ffs(~master->slot_map) - 1;
+		device->abs.seat_slot = seat_slot;
+		master->slot_map |= 1 << seat_slot;
+		notify_touch(master, time, seat_slot, x, y, WL_TOUCH_DOWN);
+		break;
+	case EVDEV_ABSOLUTE_MOTION:
+		if (device->output == NULL)
+			break;
+		transform_absolute(device, &cx, &cy);
+		weston_output_transform_coordinate(device->output,
+						   wl_fixed_from_int(cx),
+						   wl_fixed_from_int(cy),
+						   &x, &y);
+
+		if (device->seat_caps & EVDEV_SEAT_TOUCH)
+			notify_touch(master, time, device->abs.seat_slot,
+				     x, y, WL_TOUCH_MOTION);
+		else if (device->seat_caps & EVDEV_SEAT_POINTER)
+			notify_motion_absolute(master, time, x, y);
+		break;
+	case EVDEV_ABSOLUTE_TOUCH_UP:
+		seat_slot = device->abs.seat_slot;
+		master->slot_map &= ~(1 << seat_slot);
+		notify_touch(master, time, seat_slot, 0, 0, WL_TOUCH_UP);
+		break;
+	default:
+		assert(0 && "Unknown pending event type");
+	}
+
+	device->pending_event = EVDEV_NONE;
+}
+
+static void
+evdev_process_touch_button(struct evdev_device *device, int time, int value)
+{
+	if (device->pending_event != EVDEV_NONE &&
+	    device->pending_event != EVDEV_ABSOLUTE_MOTION)
+		evdev_flush_pending_event(device, time);
+
+	device->pending_event = (value ?
+				 EVDEV_ABSOLUTE_TOUCH_DOWN :
+				 EVDEV_ABSOLUTE_TOUCH_UP);
+}
+
 static inline void
 evdev_process_key(struct evdev_device *device, struct input_event *e, int time)
 {
+	/* ignore kernel key repeat */
 	if (e->value == 2)
 		return;
 
+	if (e->code == BTN_TOUCH) {
+		if (!device->is_mt)
+			evdev_process_touch_button(device, time, e->value);
+		return;
+	}
+
+	evdev_flush_pending_event(device, time);
+
 	switch (e->code) {
 	case BTN_LEFT:
 	case BTN_RIGHT:
@@ -92,34 +222,45 @@ evdev_process_key(struct evdev_device *device, struct input_event *e, int time)
 }
 
 static void
-evdev_process_touch(struct evdev_device *device, struct input_event *e)
+evdev_process_touch(struct evdev_device *device,
+		    struct input_event *e,
+		    uint32_t time)
 {
-	const int screen_width = device->output->current->width;
-	const int screen_height = device->output->current->height;
+	int screen_width, screen_height;
+
+	if (device->output == NULL)
+		return;
+
+	screen_width = device->output->current_mode->width;
+	screen_height = device->output->current_mode->height;
 
 	switch (e->code) {
 	case ABS_MT_SLOT:
+		evdev_flush_pending_event(device, time);
 		device->mt.slot = e->value;
 		break;
 	case ABS_MT_TRACKING_ID:
+		if (device->pending_event != EVDEV_NONE &&
+		    device->pending_event != EVDEV_ABSOLUTE_MT_MOTION)
+			evdev_flush_pending_event(device, time);
 		if (e->value >= 0)
-			device->pending_events |= EVDEV_ABSOLUTE_MT_DOWN;
+			device->pending_event = EVDEV_ABSOLUTE_MT_DOWN;
 		else
-			device->pending_events |= EVDEV_ABSOLUTE_MT_UP;
+			device->pending_event = EVDEV_ABSOLUTE_MT_UP;
 		break;
 	case ABS_MT_POSITION_X:
-		device->mt.x[device->mt.slot] =
+		device->mt.slots[device->mt.slot].x =
 			(e->value - device->abs.min_x) * screen_width /
-			(device->abs.max_x - device->abs.min_x) +
-			device->output->x;
-		device->pending_events |= EVDEV_ABSOLUTE_MT_MOTION;
+			(device->abs.max_x - device->abs.min_x);
+		if (device->pending_event == EVDEV_NONE)
+			device->pending_event = EVDEV_ABSOLUTE_MT_MOTION;
 		break;
 	case ABS_MT_POSITION_Y:
-		device->mt.y[device->mt.slot] =
+		device->mt.slots[device->mt.slot].y =
 			(e->value - device->abs.min_y) * screen_height /
-			(device->abs.max_y - device->abs.min_y) +
-			device->output->y;
-		device->pending_events |= EVDEV_ABSOLUTE_MT_MOTION;
+			(device->abs.max_y - device->abs.min_y);
+		if (device->pending_event == EVDEV_NONE)
+			device->pending_event = EVDEV_ABSOLUTE_MT_MOTION;
 		break;
 	}
 }
@@ -128,23 +269,28 @@ static inline void
 evdev_process_absolute_motion(struct evdev_device *device,
 			      struct input_event *e)
 {
-	const int screen_width = device->output->current->width;
-	const int screen_height = device->output->current->height;
+	int screen_width, screen_height;
+
+	if (device->output == NULL)
+		return;
+
+	screen_width = device->output->current_mode->width;
+	screen_height = device->output->current_mode->height;
 
 	switch (e->code) {
 	case ABS_X:
 		device->abs.x =
 			(e->value - device->abs.min_x) * screen_width /
-			(device->abs.max_x - device->abs.min_x) +
-			device->output->x;
-		device->pending_events |= EVDEV_ABSOLUTE_MOTION;
+			(device->abs.max_x - device->abs.min_x);
+		if (device->pending_event == EVDEV_NONE)
+			device->pending_event = EVDEV_ABSOLUTE_MOTION;
 		break;
 	case ABS_Y:
 		device->abs.y =
 			(e->value - device->abs.min_y) * screen_height /
-			(device->abs.max_y - device->abs.min_y) +
-			device->output->y;
-		device->pending_events |= EVDEV_ABSOLUTE_MOTION;
+			(device->abs.max_y - device->abs.min_y);
+		if (device->pending_event == EVDEV_NONE)
+			device->pending_event = EVDEV_ABSOLUTE_MOTION;
 		break;
 	}
 }
@@ -155,14 +301,19 @@ evdev_process_relative(struct evdev_device *device,
 {
 	switch (e->code) {
 	case REL_X:
+		if (device->pending_event != EVDEV_RELATIVE_MOTION)
+			evdev_flush_pending_event(device, time);
 		device->rel.dx += wl_fixed_from_int(e->value);
-		device->pending_events |= EVDEV_RELATIVE_MOTION;
+		device->pending_event = EVDEV_RELATIVE_MOTION;
 		break;
 	case REL_Y:
+		if (device->pending_event != EVDEV_RELATIVE_MOTION)
+			evdev_flush_pending_event(device, time);
 		device->rel.dy += wl_fixed_from_int(e->value);
-		device->pending_events |= EVDEV_RELATIVE_MOTION;
+		device->pending_event = EVDEV_RELATIVE_MOTION;
 		break;
 	case REL_WHEEL:
+		evdev_flush_pending_event(device, time);
 		switch (e->value) {
 		case -1:
 			/* Scroll down */
@@ -178,6 +329,7 @@ evdev_process_relative(struct evdev_device *device,
 		}
 		break;
 	case REL_HWHEEL:
+		evdev_flush_pending_event(device, time);
 		switch (e->value) {
 		case -1:
 			/* Scroll left */
@@ -196,86 +348,17 @@ evdev_process_relative(struct evdev_device *device,
 }
 
 static inline void
-evdev_process_absolute(struct evdev_device *device, struct input_event *e)
+evdev_process_absolute(struct evdev_device *device,
+		       struct input_event *e,
+		       uint32_t time)
 {
 	if (device->is_mt) {
-		evdev_process_touch(device, e);
+		evdev_process_touch(device, e, time);
 	} else {
 		evdev_process_absolute_motion(device, e);
 	}
 }
 
-static int
-is_motion_event(struct input_event *e)
-{
-	switch (e->type) {
-	case EV_REL:
-		switch (e->code) {
-		case REL_X:
-		case REL_Y:
-			return 1;
-		}
-		break;
-	case EV_ABS:
-		switch (e->code) {
-		case ABS_X:
-		case ABS_Y:
-		case ABS_MT_POSITION_X:
-		case ABS_MT_POSITION_Y:
-			return 1;
-		}
-	}
-
-	return 0;
-}
-
-static void
-evdev_flush_motion(struct evdev_device *device, uint32_t time)
-{
-	struct weston_seat *master = device->seat;
-
-	if (!device->pending_events)
-		return;
-
-	if (device->pending_events & EVDEV_RELATIVE_MOTION) {
-		notify_motion(master, time,
-			      master->seat.pointer->x + device->rel.dx,
-			      master->seat.pointer->y + device->rel.dy);
-		device->pending_events &= ~EVDEV_RELATIVE_MOTION;
-		device->rel.dx = 0;
-		device->rel.dy = 0;
-	}
-	if (device->pending_events & EVDEV_ABSOLUTE_MT_DOWN) {
-		notify_touch(master, time,
-			     device->mt.slot,
-			     wl_fixed_from_int(device->mt.x[device->mt.slot]),
-			     wl_fixed_from_int(device->mt.y[device->mt.slot]),
-			     WL_TOUCH_DOWN);
-		device->pending_events &= ~EVDEV_ABSOLUTE_MT_DOWN;
-		device->pending_events &= ~EVDEV_ABSOLUTE_MT_MOTION;
-	}
-	if (device->pending_events & EVDEV_ABSOLUTE_MT_MOTION) {
-		notify_touch(master, time,
-			     device->mt.slot,
-			     wl_fixed_from_int(device->mt.x[device->mt.slot]),
-			     wl_fixed_from_int(device->mt.y[device->mt.slot]),
-			     WL_TOUCH_MOTION);
-		device->pending_events &= ~EVDEV_ABSOLUTE_MT_DOWN;
-		device->pending_events &= ~EVDEV_ABSOLUTE_MT_MOTION;
-	}
-	if (device->pending_events & EVDEV_ABSOLUTE_MT_UP) {
-		notify_touch(master, time, device->mt.slot, 0, 0,
-			     WL_TOUCH_UP);
-		device->pending_events &= ~EVDEV_ABSOLUTE_MT_UP;
-	}
-	if (device->pending_events & EVDEV_ABSOLUTE_MOTION) {
-		notify_motion(master, time,
-			      wl_fixed_from_int(device->abs.x),
-			      wl_fixed_from_int(device->abs.y));
-		device->pending_events &= ~EVDEV_ABSOLUTE_MOTION;
-	}
-}
-
 static void
 fallback_process(struct evdev_dispatch *dispatch,
 		 struct evdev_device *device,
@@ -287,11 +370,14 @@ fallback_process(struct evdev_dispatch *dispatch,
 		evdev_process_relative(device, event, time);
 		break;
 	case EV_ABS:
-		evdev_process_absolute(device, event);
+		evdev_process_absolute(device, event, time);
 		break;
 	case EV_KEY:
 		evdev_process_key(device, event, time);
 		break;
+	case EV_SYN:
+		evdev_flush_pending_event(device, time);
+		break;
 	}
 }
 
@@ -326,23 +412,13 @@ evdev_process_events(struct evdev_device *device,
 	struct input_event *e, *end;
 	uint32_t time = 0;
 
-	device->pending_events = 0;
-
 	e = ev;
 	end = e + count;
 	for (e = ev; e < end; e++) {
 		time = e->time.tv_sec * 1000 + e->time.tv_usec / 1000;
 
-		/* we try to minimize the amount of notifications to be
-		 * forwarded to the compositor, so we accumulate motion
-		 * events and send as a bunch */
-		if (!is_motion_event(e))
-			evdev_flush_motion(device, time);
-
 		dispatch->interface->process(dispatch, device, e, time);
 	}
-
-	evdev_flush_motion(device, time);
 }
 
 static int
@@ -354,7 +430,7 @@ evdev_device_data(int fd, uint32_t mask, void *data)
 	int len;
 
 	ec = device->seat->compositor;
-	if (!ec->focus)
+	if (!ec->session_active)
 		return 1;
 
 	/* If the compositor is repainting, this function is called only once
@@ -369,7 +445,13 @@ evdev_device_data(int fd, uint32_t mask, void *data)
 			len = read(fd, &ev, sizeof ev);
 
 		if (len < 0 || len % sizeof ev[0] != 0) {
-			/* FIXME: call evdev_device_destroy when errno is ENODEV. */
+			if (len < 0 && errno != EAGAIN && errno != EINTR) {
+				weston_log("device %s died\n",
+					   device->devnode);
+				wl_event_source_remove(device->source);
+				device->source = NULL;
+			}
+
 			return 1;
 		}
 
@@ -388,32 +470,39 @@ evdev_configure_device(struct evdev_device *device)
 	unsigned long abs_bits[NBITS(ABS_MAX)];
 	unsigned long rel_bits[NBITS(REL_MAX)];
 	unsigned long key_bits[NBITS(KEY_MAX)];
-	int has_key, has_abs;
+	int has_abs, has_rel, has_mt;
+	int has_button, has_keyboard, has_touch;
 	unsigned int i;
 
-	has_key = 0;
+	has_rel = 0;
 	has_abs = 0;
-	device->caps = 0;
+	has_mt = 0;
+	has_button = 0;
+	has_keyboard = 0;
+	has_touch = 0;
 
 	ioctl(device->fd, EVIOCGBIT(0, sizeof(ev_bits)), ev_bits);
 	if (TEST_BIT(ev_bits, EV_ABS)) {
-		has_abs = 1;
-
 		ioctl(device->fd, EVIOCGBIT(EV_ABS, sizeof(abs_bits)),
 		      abs_bits);
+
 		if (TEST_BIT(abs_bits, ABS_X)) {
 			ioctl(device->fd, EVIOCGABS(ABS_X), &absinfo);
 			device->abs.min_x = absinfo.minimum;
 			device->abs.max_x = absinfo.maximum;
-			device->caps |= EVDEV_MOTION_ABS;
+			has_abs = 1;
 		}
 		if (TEST_BIT(abs_bits, ABS_Y)) {
 			ioctl(device->fd, EVIOCGABS(ABS_Y), &absinfo);
 			device->abs.min_y = absinfo.minimum;
 			device->abs.max_y = absinfo.maximum;
-			device->caps |= EVDEV_MOTION_ABS;
+			has_abs = 1;
 		}
-		if (TEST_BIT(abs_bits, ABS_MT_SLOT)) {
+                /* We only handle the slotted Protocol B in weston.
+                   Devices with ABS_MT_POSITION_* but not ABS_MT_SLOT
+                   require mtdev for conversion. */
+		if (TEST_BIT(abs_bits, ABS_MT_POSITION_X) &&
+		    TEST_BIT(abs_bits, ABS_MT_POSITION_Y)) {
 			ioctl(device->fd, EVIOCGABS(ABS_MT_POSITION_X),
 			      &absinfo);
 			device->abs.min_x = absinfo.minimum;
@@ -423,66 +512,79 @@ evdev_configure_device(struct evdev_device *device)
 			device->abs.min_y = absinfo.minimum;
 			device->abs.max_y = absinfo.maximum;
 			device->is_mt = 1;
-			device->mt.slot = 0;
-			device->caps |= EVDEV_TOUCH;
+			has_touch = 1;
+			has_mt = 1;
+
+			if (!TEST_BIT(abs_bits, ABS_MT_SLOT)) {
+				device->mtdev = mtdev_new_open(device->fd);
+				if (!device->mtdev) {
+					weston_log("mtdev required but failed to open for %s\n",
+						   device->devnode);
+					return 0;
+				}
+				device->mt.slot = device->mtdev->caps.slot.value;
+			} else {
+				ioctl(device->fd, EVIOCGABS(ABS_MT_SLOT),
+				      &absinfo);
+				device->mt.slot = absinfo.value;
+			}
 		}
 	}
 	if (TEST_BIT(ev_bits, EV_REL)) {
 		ioctl(device->fd, EVIOCGBIT(EV_REL, sizeof(rel_bits)),
 		      rel_bits);
 		if (TEST_BIT(rel_bits, REL_X) || TEST_BIT(rel_bits, REL_Y))
-			device->caps |= EVDEV_MOTION_REL;
+			has_rel = 1;
 	}
 	if (TEST_BIT(ev_bits, EV_KEY)) {
-		has_key = 1;
 		ioctl(device->fd, EVIOCGBIT(EV_KEY, sizeof(key_bits)),
 		      key_bits);
 		if (TEST_BIT(key_bits, BTN_TOOL_FINGER) &&
 		    !TEST_BIT(key_bits, BTN_TOOL_PEN) &&
-		    has_abs)
+		    (has_abs || has_mt)) {
 			device->dispatch = evdev_touchpad_create(device);
+			weston_log("input device %s, %s is a touchpad\n",
+				   device->devname, device->devnode);
+		}
 		for (i = KEY_ESC; i < KEY_MAX; i++) {
 			if (i >= BTN_MISC && i < KEY_OK)
 				continue;
 			if (TEST_BIT(key_bits, i)) {
-				device->caps |= EVDEV_KEYBOARD;
+				has_keyboard = 1;
 				break;
 			}
 		}
-		for (i = BTN_MISC; i < KEY_OK; i++) {
+		if (TEST_BIT(key_bits, BTN_TOUCH))
+			has_touch = 1;
+		for (i = BTN_MISC; i < BTN_JOYSTICK; i++) {
 			if (TEST_BIT(key_bits, i)) {
-				device->caps |= EVDEV_BUTTON;
+				has_button = 1;
 				break;
 			}
 		}
 	}
-	if (TEST_BIT(ev_bits, EV_LED)) {
-		device->caps |= EVDEV_KEYBOARD;
-	}
+	if (TEST_BIT(ev_bits, EV_LED))
+		has_keyboard = 1;
 
-	/* This rule tries to catch accelerometer devices and opt out. We may
-	 * want to adjust the protocol later adding a proper event for dealing
-	 * with accelerometers and implement here accordingly */
-	if (has_abs && !has_key && !device->is_mt) {
-		weston_log("input device %s, %s "
-			   "ignored: unsupported device type\n",
-			   device->devname, device->devnode);
-		return -1;
-	}
-
-	if ((device->caps &
-	     (EVDEV_MOTION_ABS | EVDEV_MOTION_REL | EVDEV_BUTTON))) {
+	if ((has_abs || has_rel) && has_button) {
 		weston_seat_init_pointer(device->seat);
-		weston_log("input device %s, %s is a pointer\n",
-			   device->devname, device->devnode);
+		device->seat_caps |= EVDEV_SEAT_POINTER;
+		weston_log("input device %s, %s is a pointer caps =%s%s%s\n",
+			   device->devname, device->devnode,
+			   has_abs ? " absolute-motion" : "",
+			   has_rel ? " relative-motion": "",
+			   has_button ? " button" : "");
 	}
-	if ((device->caps & EVDEV_KEYBOARD)) {
-		weston_seat_init_keyboard(device->seat, NULL);
+	if (has_keyboard) {
+		if (weston_seat_init_keyboard(device->seat, NULL) < 0)
+			return -1;
+		device->seat_caps |= EVDEV_SEAT_KEYBOARD;
 		weston_log("input device %s, %s is a keyboard\n",
 			   device->devname, device->devnode);
 	}
-	if ((device->caps & EVDEV_TOUCH)) {
+	if (has_touch && !has_button) {
 		weston_seat_init_touch(device->seat);
+		device->seat_caps |= EVDEV_SEAT_TOUCH;
 		weston_log("input device %s, %s is a touch device\n",
 			   device->devname, device->devnode);
 	}
@@ -490,6 +592,39 @@ evdev_configure_device(struct evdev_device *device)
 	return 0;
 }
 
+static void
+notify_output_destroy(struct wl_listener *listener, void *data)
+{
+	struct evdev_device *device =
+		container_of(listener,
+			     struct evdev_device, output_destroy_listener);
+	struct weston_compositor *c = device->seat->compositor;
+	struct weston_output *output;
+
+	if (!device->output_name && !wl_list_empty(&c->output_list)) {
+		output = container_of(c->output_list.next,
+				      struct weston_output, link);
+		evdev_device_set_output(device, output);
+	} else {
+		device->output = NULL;
+	}
+}
+
+void
+evdev_device_set_output(struct evdev_device *device,
+			struct weston_output *output)
+{
+	if (device->output_destroy_listener.notify) {
+		wl_list_remove(&device->output_destroy_listener.link);
+		device->output_destroy_listener.notify = NULL;
+	}
+
+	device->output = output;
+	device->output_destroy_listener.notify = notify_output_destroy;
+	wl_signal_add(&output->destroy_signal,
+		      &device->output_destroy_listener);
+}
+
 struct evdev_device *
 evdev_device_create(struct weston_seat *seat, const char *path, int device_fd)
 {
@@ -497,16 +632,13 @@ evdev_device_create(struct weston_seat *seat, const char *path, int device_fd)
 	struct weston_compositor *ec;
 	char devname[256] = "unknown";
 
-	device = malloc(sizeof *device);
+	device = zalloc(sizeof *device);
 	if (device == NULL)
 		return NULL;
-	memset(device, 0, sizeof *device);
 
 	ec = seat->compositor;
-	device->output =
-		container_of(ec->output_list.next, struct weston_output, link);
-
 	device->seat = seat;
+	device->seat_caps = 0;
 	device->is_mt = 0;
 	device->mtdev = NULL;
 	device->devnode = strdup(path);
@@ -515,40 +647,37 @@ evdev_device_create(struct weston_seat *seat, const char *path, int device_fd)
 	device->rel.dy = 0;
 	device->dispatch = NULL;
 	device->fd = device_fd;
+	device->pending_event = EVDEV_NONE;
+	wl_list_init(&device->link);
 
 	ioctl(device->fd, EVIOCGNAME(sizeof(devname)), devname);
+	devname[sizeof(devname) - 1] = '\0';
 	device->devname = strdup(devname);
 
 	if (evdev_configure_device(device) == -1)
-		goto err1;
+		goto err;
+
+	if (device->seat_caps == 0) {
+		evdev_device_destroy(device);
+		return EVDEV_UNHANDLED_DEVICE;
+	}
 
 	/* If the dispatch was not set up use the fallback. */
 	if (device->dispatch == NULL)
 		device->dispatch = fallback_dispatch_create();
 	if (device->dispatch == NULL)
-		goto err1;
-
-
-	if (device->is_mt) {
-		device->mtdev = mtdev_new_open(device->fd);
-		if (!device->mtdev)
-			weston_log("mtdev failed to open for %s\n", path);
-	}
+		goto err;
 
 	device->source = wl_event_loop_add_fd(ec->input_loop, device->fd,
 					      WL_EVENT_READABLE,
 					      evdev_device_data, device);
 	if (device->source == NULL)
-		goto err2;
+		goto err;
 
 	return device;
 
-err2:
-	device->dispatch->interface->destroy(device->dispatch);
-err1:
-	free(device->devname);
-	free(device->devnode);
-	free(device);
+err:
+	evdev_device_destroy(device);
 	return NULL;
 }
 
@@ -557,17 +686,28 @@ evdev_device_destroy(struct evdev_device *device)
 {
 	struct evdev_dispatch *dispatch;
 
+	if (device->seat_caps & EVDEV_SEAT_POINTER)
+		weston_seat_release_pointer(device->seat);
+	if (device->seat_caps & EVDEV_SEAT_KEYBOARD)
+		weston_seat_release_keyboard(device->seat);
+	if (device->seat_caps & EVDEV_SEAT_TOUCH)
+		weston_seat_release_touch(device->seat);
+
 	dispatch = device->dispatch;
 	if (dispatch)
 		dispatch->interface->destroy(dispatch);
 
-	wl_event_source_remove(device->source);
+	if (device->source)
+		wl_event_source_remove(device->source);
+	if (device->output)
+		wl_list_remove(&device->output_destroy_listener.link);
 	wl_list_remove(&device->link);
 	if (device->mtdev)
 		mtdev_close_delete(device->mtdev);
 	close(device->fd);
 	free(device->devname);
 	free(device->devnode);
+	free(device->output_name);
 	free(device);
 }
 
@@ -583,7 +723,7 @@ evdev_notify_keyboard_focus(struct weston_seat *seat,
 	uint32_t *k;
 	int ret;
 
-	if (!seat->seat.keyboard)
+	if (!seat->keyboard_device_count > 0)
 		return;
 
 	memset(all_keys, 0, sizeof all_keys);
