@@ -120,8 +120,10 @@ move_client(struct client *client, int x, int y);
 struct wl_callback *
 frame_callback_set(struct wl_surface *surface, int *done);
 
-void
-frame_callback_wait(struct client *client, int *done);
+int
+frame_callback_wait_nofail(struct client *client, int *done);
+
+#define frame_callback_wait(c, d) assert(frame_callback_wait_nofail((c), (d)))
 
 int
 get_n_egl_buffers(struct client *client);
@@ -129,4 +131,8 @@ get_n_egl_buffers(struct client *client);
 void
 skip(const char *fmt, ...);
 
+void
+expect_protocol_error(struct client *client,
+		      const struct wl_interface *intf, uint32_t code);
+
 #endif
