@@ -93,9 +93,9 @@ test_surface_configure(struct weston_surface *surface, int32_t sx, int32_t sy)
 	struct weston_test_surface *test_surface = surface->configure_private;
 	struct weston_test *test = test_surface->test;
 
-	if (wl_list_empty(&test_surface->view->layer_link))
-		wl_list_insert(&test->layer.view_list,
-			       &test_surface->view->layer_link);
+	if (wl_list_empty(&test_surface->view->layer_link.link))
+		weston_layer_entry_insert(&test->layer.view_list,
+					  &test_surface->view->layer_link);
 
 	weston_view_set_position(test_surface->view,
 				 test_surface->x, test_surface->y);
@@ -251,6 +251,11 @@ bind_test(struct wl_client *client, void *data, uint32_t version, uint32_t id)
 	struct wl_resource *resource;
 
 	resource = wl_resource_create(client, &wl_test_interface, 1, id);
+	if (!resource) {
+		wl_client_post_no_memory(client);
+		return;
+	}
+
 	wl_resource_set_implementation(resource,
 				       &test_implementation, test, NULL);
 
